// This is a part of the SWF Studio Plugin SDK Library.
// Copyright (C) 2004 Northcode Inc.
// All rights reserved.
//
// This source code is only intended for use by SWF Studio Plugin Developers,
// and its use is subject to the restrictions defined in the
// SWF Studio Plugin Developer Licence Agreement.
// 
// For information, consult the SWF Studion Plugin SDK documentation,
// or contact info@northcode.com

#ifndef __NORTHCODE_PLUGIN_BASE_H__
#define __NORTHCODE_PLUGIN_BASE_H__


// This header file uses BSTRs, so let's include the definition:
	#include <atlbase.h> // for BSTR definition
	typedef const BSTR 	CBSTR;

	extern HINSTANCE g_hInstance;

// To ensure the plugin exposes all the right calls, and to prevent linker warnings:
	#pragma comment(linker, "/include:_DllMain@12")
	#pragma comment(linker, "/nodefaultlib:libcd")
	#pragma comment(linker, "/nodefaultlib:libc")



// Macros used in the Plugin command declaration
	#define DECLARE_COMMAND(_command)																					\
			friend HRESULT InstanceCommand_Stub_##_command(void *pInstance, NCParameters *pNCCommandParameters);		\
			HRESULT _command(NCParameters *pNCCommandParameters);

// Macros used in the Plugin class implementation
	// if multiple instances are permitted, use this:
	#define IMPLEMENT_PLUGIN(_myclass)																					\
			void* On_NC_Construct(CBSTR bsAlias) { return new _myclass; }												\
			void On_NC_Destruct(void* pDataBlock) { delete (_myclass*)(pDataBlock); }									\
			int On_NC_GetMaxInstanceCount(void) { return 2147483647; }
	
	// if only 1 instance is permitted at a time, use this:
	#define IMPLEMENT_SINGLETON_PLUGIN(_myclass)																		\
			void* On_NC_Construct(CBSTR bsAlias) { return new _myclass; }												\
			void On_NC_Destruct(void* pDataBlock) { delete (_myclass*)(pDataBlock); }									\
			int On_NC_GetMaxInstanceCount(void) { return 1; }

	// Macro used for defining Plugin properties
	// Note that the Key must only contain variable-name-safe characters, eg. alpha-numerics and
	// underscores are ok, periods and colons are not.
	#define DEFINE_PROPERTY(key, value)																					\
			static NCPluginProperty NCPluginProperty_##key(L# key, value)

// Macros and classes used in the implementation of Commands.
	#define IMPLEMENT_COMMAND(_class, _command)																			\
			HRESULT InstanceCommand_Stub_##_command(void *pInstance, NCParameters *pNCCommandParameters)				\
					{ return ((_class*)(pInstance))->_command(pNCCommandParameters); }									\
			static NCPluginCommandEntry	InstanceCommandEntry_##_command(L# _command, InstanceCommand_Stub_##_command);	\
			HRESULT _class::_command(NCParameters *pNCCommandParameters)

	#define IMPLEMENT_COMMAND_ASYNC(_class, _command)																	\
			HRESULT InstanceCommand_Stub_##_command(void *pInstance, NCParameters *pNCCommandParameters)				\
					{ return ((_class*)(pInstance))->_command(pNCCommandParameters); }									\
			static NCPluginCommandEntry	InstanceCommandEntry_##_command(L# _command, InstanceCommand_Stub_##_command, TRUE);	\
			HRESULT _class::_command(NCParameters *pNCCommandParameters)

// Macros used for auto-documentation of commands
	#define DECLARE_PARAMETERS(_myclass, _command, _description)														\
				void InstanceCommandParameters_Stub_##_command(NCManifest&);											\
				static NCManifest InstanceCommandParameters_##_command(													\
															L# _command,												\
															InstanceCommandParameters_Stub_##_command,					\
															_description);												\
				void InstanceCommandParameters_Stub_##_command(NCManifest &params)										\
				{

		#define IN_PARAMETER(_name, _description, _values)																\
					params.AddParameter(L# _name,																		\
										_description,																	\
										FALSE,																			\
										NCManifest::in,																	\
										NCManifest::required,															\
										L"",																			\
										_values);

		#define IN_PARAMETER_OPTIONAL(_name, _default_value, _description, _values)										\
					params.AddParameter(L# _name,																		\
										_description,																	\
										FALSE,																			\
										NCManifest::in,																	\
										NCManifest::optional,															\
										_default_value,																	\
										_values);

		#define IN_PARAMETER_MONIKER(_name, _description, _values)														\
					params.AddParameter(L# _name,																		\
										_description,																	\
										TRUE,																			\
										NCManifest::in,																	\
										NCManifest::required,															\
										L"",																			\
										_values);

		#define IN_PARAMETER_MONIKER_OPTIONAL(_name, _default_value, _description, _values)								\
					params.AddParameter(L# _name,																		\
										_description,																	\
										TRUE,																			\
										NCManifest::in,																	\
										NCManifest::optional,															\
										_default_value,																	\
										_values);

		#define OUT_PARAMETER(_name, _description)																		\
					params.AddParameter(L# _name,																		\
										_description,																	\
										FALSE,																			\
										NCManifest::out,																\
										NCManifest::required,															\
										L"", L"");

		#define ERROR_CODE(_code, _description)																			\
					params.AddErrorCode(_code,																			\
										_description);

		#define DEPRECATED(_replacement, _version)																		\
					params.Deprecated(L# _replacement, _version);

		#define ADDED(_version)																							\
					params.Added(_version);

		#define EXCLUDED_FROM_AS3																						\
					params.ExcludedFromAS3();

		#define HIDDEN																									\
					params.Hidden();


	#define END_PARAMETERS																								\
				}

	class NCParameters
	{
		public:
			NCParameters(void);
			~NCParameters();

			const LPWSTR	GetParameter(const LPWSTR szParamName, HRESULT *pStatus = NULL);
			void			SetParameter(const LPWSTR szParamName, const LPWSTR szValue);
			void			SetParameter(const LPWSTR szParamName, int nValue);
			void			SetParameter(const LPWSTR szParamName, __int64 nValue);

			void			SendNotify(const LPWSTR szFlashVariable, const LPWSTR szValue);
			void			SendNotify(const LPWSTR szFlashVariable, int nValue);
			void			SendNotify(const LPWSTR szFlashVariable, __int64 nValue);

			// DO NOT USE THE TWO FOLLOWING METHODS.  THEY ARE UNSUPPORTED
			static void	UnsafeSendNotify(const LPWSTR szFlashVariable, const LPWSTR szValue);
			static void	UnsafeSendNotify(const LPWSTR szFlashVariable, int nValue);
			static void	UnsafeSendNotify(const LPWSTR szFlashVariable, __int64 nValue);

		// The following implementations "assign" ownership of the BSTRs to the called function.
		// The caller must relinquish ownership, ie. not modify or delete the BSTRs.
		// By assigning ownership, many memory copies are eliminated, improved memory efficiency and performance.
		// The BSTRs MUST be real, SysAllocString'ed BSTRs, since the API will call GetSysStringLength to determine the string size.
		public:
			void			SetParameter_Assign(BSTR bstrParamName, BSTR bstrValue);
			void			SetParameter_Assign(BSTR bstrParamName, int nValue);
			void			SetParameter_Assign(BSTR bstrParamName, __int64 nValue);

			void			SendNotify_Assign(BSTR bstrFlashVariable, BSTR bstrValue);
			void			SendNotify_Assign(BSTR bstrFlashVariable, int nValue);
			void			SendNotify_Assign(BSTR bstrFlashVariable, __int64 nValue);

			// DO NOT USE THE TWO FOLLOWING METHODS.  THEY ARE UNSUPPORTED
			static void	UnsafeSendNotify_Assign(BSTR bstrFlashVariable, BSTR bstrValue);
			static void	UnsafeSendNotify_Assign(BSTR bstrFlashVariable, int nValue);
			static void	UnsafeSendNotify_Assign(BSTR bstrFlashVariable, __int64 nValue);

		protected:
			void	*m_pData;
	};

// Error Codes:
#define NC_ERROR_GENERIC		-1
#define NC_ERROR_BAD_ALIAS		-2
#define NC_ERROR_BAD_COMMAND	-3
#define NC_ERROR_COM			-4
#define NC_ERROR_ASYNCHRONOUS	-5
#define NC_ERROR_COMMAND_FAILED -6

// Debugging 
#ifdef _DEBUG
	#define NCTRACE		NCTraceToStudio
#else
	#define NCTRACE		1 ? (void)0 : NCTraceToStudio
#endif
void NCTraceToStudio(const LPWSTR lpszFormat, ...);


// For Northcode's internal use.
	class NCInitializer
	{
		public:
			NCInitializer(void);
			~NCInitializer();
		protected:
			static int	m_nRefCount;
	};

	class NCPluginProperty : public NCInitializer
	{
		public:
			NCPluginProperty(const LPWSTR szProperty, const LPWSTR szValue);
			~NCPluginProperty();

			static CBSTR GetProperty(const LPWSTR szProperty, const LPWSTR szDefault = NULL, HRESULT *pResult = NULL);
	};

	class NCPluginCommandEntry : public NCInitializer
	{
		public:
			NCPluginCommandEntry(const LPWSTR szCommand, void* pfnFunction, BOOL bAsynchronous = FALSE);
			~NCPluginCommandEntry();
	};

	class NCManifest : public NCInitializer
	{
		public:
			enum	Direction	{ in, out };
			enum	Presence	{ required, optional };
			typedef void (*HELPER) (NCManifest&) ;


			NCManifest(const LPWSTR, HELPER, const LPWSTR);
			~NCManifest();
			void AddParameter(const LPWSTR, const LPWSTR, BOOL, Direction, Presence, const LPWSTR, const LPWSTR);
			void AddErrorCode(int, const LPWSTR);
			void Deprecated(const LPWSTR, const LPWSTR);
			void Added(const LPWSTR);
			void ExcludedFromAS3(BOOL bExcluded = TRUE);
			void Hidden(BOOL bHidden = TRUE);

		protected:
			void	*m_pData;
	};

#endif __NORTHCODE_PLUGIN_BASE_H__